@@ -78,7 +78,7 @@ static void hinawa_fw_resp_class_init(HinawaFwRespClass *klass)
 		g_signal_new("requested",
 			     G_OBJECT_CLASS_TYPE(klass),
 			     G_SIGNAL_RUN_LAST,
-			     0,
+			     G_STRUCT_OFFSET(HinawaFwRespClass, requested),
 			     NULL, NULL,
 			     hinawa_sigs_marshal_ENUM__ENUM,
 			     HINAWA_TYPE_FW_RCODE, 1, HINAWA_TYPE_FW_TCODE);
@@ -186,6 +186,8 @@ void hinawa_fw_resp_unregister(HinawaFwResp *self)
  * @frame: (element-type guint8)(array length=length)(out): a 8bit array for
  * 	   response frame.
  * @length: (out): The length of bytes for the frame.
+ *
+ * Retrieve byte frame to be requested.
  */
 void hinawa_fw_resp_get_req_frame(HinawaFwResp *self, const guint8 **frame,
 				  guint *length)
@@ -207,6 +209,8 @@ void hinawa_fw_resp_get_req_frame(HinawaFwResp *self, const guint8 **frame,
  * @frame: (element-type guint8)(array length=length): a 8bit array for response
  * 	   frame.
  * @length: The length of bytes for the frame.
+ *
+ * Register byte frame as response.
  */
 void hinawa_fw_resp_set_resp_frame(HinawaFwResp *self, guint8 *frame,
 				   guint length)
