@@ -38,6 +38,8 @@ typedef struct {
 	GSource src;
 	HinawaFwUnit *unit;
 	gpointer tag;
+	unsigned int len;
+	void *buf;
 } FwUnitSource;
 
 struct _HinawaFwUnitPrivate {
@@ -48,8 +50,6 @@ struct _HinawaFwUnitPrivate {
 	unsigned int config_rom_length;
 	struct fw_cdev_event_bus_reset generation;
 
-	unsigned int len;
-	void *buf;
 	FwUnitSource *src;
 };
 G_DEFINE_TYPE_WITH_PRIVATE(HinawaFwUnit, hinawa_fw_unit, G_TYPE_OBJECT)
@@ -206,7 +206,7 @@ static void hinawa_fw_unit_class_init(HinawaFwUnitClass *klass)
 		g_signal_new("bus-update",
 			     G_OBJECT_CLASS_TYPE(klass),
 			     G_SIGNAL_RUN_LAST,
-			     0,
+			     G_STRUCT_OFFSET(HinawaFwUnitClass, bus_update),
 			     NULL, NULL,
 			     g_cclosure_marshal_VOID__VOID,
 			     G_TYPE_NONE, 0, G_TYPE_NONE);
@@ -222,7 +222,7 @@ static void hinawa_fw_unit_class_init(HinawaFwUnitClass *klass)
 		g_signal_new("disconnected",
 			     G_OBJECT_CLASS_TYPE(klass),
 			     G_SIGNAL_RUN_LAST,
-			     0,
+			     G_STRUCT_OFFSET(HinawaFwUnitClass, disconnected),
 			     NULL, NULL,
 			     g_cclosure_marshal_VOID__VOID,
 			     G_TYPE_NONE, 0, G_TYPE_NONE);
@@ -268,6 +268,8 @@ static void update_info(HinawaFwUnit *self, GError **exception)
  * @self: A #HinawaFwUnit
  * @path: A path to Linux FireWire character device
  * @exception: A #GError
+ *
+ * Open Linux FireWire character device to operate for node on IEEE 1394 bus.
  */
 void hinawa_fw_unit_open(HinawaFwUnit *self, gchar *path, GError **exception)
 {
@@ -294,6 +296,8 @@ void hinawa_fw_unit_open(HinawaFwUnit *self, gchar *path, GError **exception)
  * @self: A #HinawaFwUnit
  * @length: (out) (optional): the number of bytes consists of the config rom
  *
+ * Get cached content of configuration ROM.
+ *
  * Returns: (element-type guint8) (array length=length) (transfer none): config rom image
  */
 const guint8 *hinawa_fw_unit_get_config_rom(HinawaFwUnit *self, guint *length)
@@ -369,11 +373,11 @@ static gboolean check_src(GSource *gsrc)
 		goto end;
 	}
 
-	len = read(priv->fd, priv->buf, priv->len);
+	len = read(priv->fd, src->buf, src->len);
 	if (len <= 0)
 		goto end;
 
-	common = (struct fw_cdev_event_common *)priv->buf;
+	common = (struct fw_cdev_event_common *)src->buf;
 
 	if (HINAWA_IS_FW_UNIT(common->closure) &&
 	    common->type == FW_CDEV_EVENT_BUS_RESET)
@@ -399,6 +403,13 @@ static gboolean dispatch_src(GSource *src, GSourceFunc callback,
 	return TRUE;
 }
 
+static void finalize_src(GSource *gsrc)
+{
+	FwUnitSource *src = (FwUnitSource *)gsrc;
+
+	g_free(src->buf);
+}
+
 /**
  * hinawa_fw_unit_listen:
  * @self: A #HinawaFwUnit
@@ -412,10 +423,11 @@ void hinawa_fw_unit_listen(HinawaFwUnit *self, GError **exception)
 		.prepare	= prepare_src,
 		.check		= check_src,
 		.dispatch	= dispatch_src,
-		.finalize	= NULL,
+		.finalize	= finalize_src,
 	};
 	HinawaFwUnitPrivate *priv;
 	void *buf;
+	unsigned int len;
 	GSource *src;
 
 	g_return_if_fail(HINAWA_IS_FW_UNIT(self));
@@ -425,7 +437,8 @@ void hinawa_fw_unit_listen(HinawaFwUnit *self, GError **exception)
 	 * MEMO: allocate one page because we cannot assume the size of
 	 * transaction frame.
 	 */
-	buf = g_malloc0(getpagesize());
+	len = sysconf(_SC_PAGESIZE);
+	buf = g_malloc0(len);
 	if (buf == NULL) {
 		raise(exception, ENOMEM);
 		return;
@@ -442,22 +455,19 @@ void hinawa_fw_unit_listen(HinawaFwUnit *self, GError **exception)
 	g_source_set_priority(src, G_PRIORITY_HIGH_IDLE);
 	g_source_set_can_recurse(src, TRUE);
 
+	((FwUnitSource *)src)->len = len;
+	((FwUnitSource *)src)->buf = buf;
 	((FwUnitSource *)src)->unit = self;
-	priv->src = (FwUnitSource *)src;
-	priv->buf = buf;
-	priv->len = getpagesize();
-
 	((FwUnitSource *)src)->tag =
-		hinawa_context_add_src(HINAWA_CONTEXT_TYPE_FW, src, priv->fd,
-				       G_IO_IN, exception);
+				g_source_add_unix_fd(src, priv->fd, G_IO_IN);
+
+	hinawa_context_add_src(HINAWA_CONTEXT_TYPE_FW, src, exception);
 	if (*exception != NULL) {
-		g_free(buf);
 		g_source_destroy(src);
-		priv->buf = NULL;
-		priv->len = 0;
-		priv->src = NULL;
 		return;
 	}
+
+	priv->src = (FwUnitSource *)src;
 }
 
 /**
@@ -469,18 +479,21 @@ void hinawa_fw_unit_listen(HinawaFwUnit *self, GError **exception)
 void hinawa_fw_unit_unlisten(HinawaFwUnit *self)
 {
 	HinawaFwUnitPrivate *priv;
+	GSource *gsrc;
 
 	g_return_if_fail(HINAWA_IS_FW_UNIT(self));
 	priv = hinawa_fw_unit_get_instance_private(self);
 
 	if (priv->src == NULL)
 		return;
+	gsrc = (GSource *)priv->src;
+
+	g_source_remove_unix_fd(gsrc, priv->src->tag);
+
+	hinawa_context_remove_src(HINAWA_CONTEXT_TYPE_FW, gsrc);
+
+	g_source_destroy(gsrc);
 
-	hinawa_context_remove_src(HINAWA_CONTEXT_TYPE_FW, (GSource *)priv->src);
 	g_free(priv->src);
 	priv->src = NULL;
-
-	g_free(priv->buf);
-	priv->buf = NULL;
-	priv->len = 0;
 }
