@@ -11,7 +11,7 @@ enum hinawa_context_type {
 	HINAWA_CONTEXT_TYPE_COUNT,
 };
 
-gpointer hinawa_context_add_src(enum hinawa_context_type type, GSource *src,
-				gint fd, GIOCondition event, GError **exception);
+void hinawa_context_add_src(enum hinawa_context_type type, GSource *src,
+			    GError **exception);
 void hinawa_context_remove_src(enum hinawa_context_type type, GSource *src);
 #endif
