@@ -68,7 +68,7 @@ static void hinawa_snd_dice_class_init(HinawaSndDiceClass *klass)
 		g_signal_new("notified",
 			     G_OBJECT_CLASS_TYPE(klass),
 			     G_SIGNAL_RUN_LAST,
-			     0,
+			     G_STRUCT_OFFSET(HinawaSndDiceClass, notified),
 			     NULL, NULL,
 			     g_cclosure_marshal_VOID__ULONG,
 			     G_TYPE_NONE, 1, G_TYPE_ULONG);
