@@ -4,6 +4,7 @@
 #include <glib.h>
 #include <glib-object.h>
 #include "snd_unit.h"
+#include "hinawa_sigs_marshal.h"
 
 G_BEGIN_DECLS
 
@@ -41,13 +42,26 @@ struct _HinawaSndTscm {
 
 struct _HinawaSndTscmClass {
 	HinawaSndUnitClass parent_class;
+
+	/**
+	 * HinawaSndTscmClass::control:
+	 * @self: A #HinawaSndTscm
+	 * @index: the numerical index on image of status and control info.
+	 * @before: the value of info before changed.
+	 * @after: the value of info after changed.
+	 *
+	 * When TASCAM FireWire unit transfer control message, the ::control
+	 * signal is emitted.
+	 */
+	void (*control)(HinawaSndTscm *self, guint index, guint before,
+			guint after);
 };
 
 GType hinawa_snd_tscm_get_type(void) G_GNUC_CONST;
 
 void hinawa_snd_tscm_open(HinawaSndTscm *self, gchar *path, GError **exception);
 
-const guint32 *const hinawa_snd_tscm_get_state(HinawaSndTscm *self,
-					       GError **exception);
+const guint32 *hinawa_snd_tscm_get_state(HinawaSndTscm *self,
+					 GError **exception);
 
 #endif
