@@ -33,6 +33,8 @@ typedef struct {
 	GSource src;
 	HinawaSndUnit *unit;
 	gpointer tag;
+	void *buf;
+	unsigned int len;
 } SndUnitSource;
 
 struct _HinawaSndUnitPrivate {
@@ -41,11 +43,7 @@ struct _HinawaSndUnitPrivate {
 
 	gboolean streaming;
 
-	void *buf;
-	unsigned int len;
 	SndUnitSource *src;
-
-	HinawaFwReq *req;
 };
 G_DEFINE_TYPE_WITH_PRIVATE(HinawaSndUnit, hinawa_snd_unit, HINAWA_TYPE_FW_UNIT)
 
@@ -113,7 +111,6 @@ static void snd_unit_finalize(GObject *obj)
 	hinawa_snd_unit_unlisten(self);
 
 	close(priv->fd);
-	g_clear_object(&priv->req);
 
 	G_OBJECT_CLASS(hinawa_snd_unit_parent_class)->finalize(obj);
 }
@@ -175,7 +172,7 @@ static void hinawa_snd_unit_class_init(HinawaSndUnitClass *klass)
 		g_signal_new("lock-status",
 			     G_OBJECT_CLASS_TYPE(klass),
 			     G_SIGNAL_RUN_LAST,
-			     0,
+			     G_STRUCT_OFFSET(HinawaSndUnitClass, lock_status),
 			     NULL, NULL,
 			     g_cclosure_marshal_VOID__BOOLEAN,
 			     G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
@@ -218,8 +215,6 @@ void hinawa_snd_unit_open(HinawaSndUnit *self, gchar *path, GError **exception)
 	hinawa_fw_unit_open(&self->parent_instance, fw_cdev, exception);
 	if (*exception != NULL)
 		goto end;
-
-	priv->req = g_object_new(HINAWA_TYPE_FW_REQ, "timeout", 40, NULL);
 end:
 	if (*exception != NULL)
 		close(priv->fd);
@@ -338,43 +333,43 @@ static gboolean check_src(GSource *gsrc)
 		goto end;
 	}
 
-	len = read(priv->fd, priv->buf, priv->len);
+	len = read(priv->fd, src->buf, src->len);
 	if (len <= 0)
 		goto end;
 
-	common = (struct snd_firewire_event_common *)priv->buf;
+	common = (struct snd_firewire_event_common *)src->buf;
 
 	if (common->type == SNDRV_FIREWIRE_EVENT_LOCK_STATUS)
-		handle_lock_event(unit, priv->buf, len);
+		handle_lock_event(unit, src->buf, len);
 #if HAVE_SND_DICE
 	else if (HINAWA_IS_SND_DICE(unit) &&
 		 common->type == SNDRV_FIREWIRE_EVENT_DICE_NOTIFICATION)
 		hinawa_snd_dice_handle_notification(HINAWA_SND_DICE(unit),
-						    priv->buf, len);
+						    src->buf, len);
 #endif
 #if HAVE_SND_EFW
 	else if (HINAWA_IS_SND_EFW(unit) &&
 		 common->type == SNDRV_FIREWIRE_EVENT_EFW_RESPONSE)
 		hinawa_snd_efw_handle_response(HINAWA_SND_EFW(unit),
-					       priv->buf, len);
+					       src->buf, len);
 #endif
 #if HAVE_SND_DG00X
 	else if (HINAWA_IS_SND_DG00X(unit) &&
 		 common->type == SNDRV_FIREWIRE_EVENT_DIGI00X_MESSAGE)
 		hinawa_snd_dg00x_handle_msg(HINAWA_SND_DG00X(unit),
-					    priv->buf, len);
+					    src->buf, len);
 #endif
 #if HAVE_SND_MOTU
 	else if (HINAWA_IS_SND_MOTU(unit) &&
 		 common->type == SNDRV_FIREWIRE_EVENT_MOTU_NOTIFICATION)
 		hinawa_snd_motu_handle_notification(HINAWA_SND_MOTU(unit),
-						    priv->buf, len);
+						    src->buf, len);
 #endif
 #if HAVE_SND_TSCM
 	else if (HINAWA_IS_SND_TSCM(unit) &&
 		 common->type == SNDRV_FIREWIRE_EVENT_TASCAM_CONTROL)
 		hinawa_snd_tscm_handle_control(HINAWA_SND_TSCM(unit),
-						    priv->buf, len);
+						    src->buf, len);
 #endif
 end:
 	/* Don't go to dispatch, then continue to process this source. */
@@ -388,6 +383,13 @@ static gboolean dispatch_src(GSource *src, GSourceFunc callback,
 	return TRUE;
 }
 
+static void finalize_src(GSource *gsrc)
+{
+	SndUnitSource *src = (SndUnitSource *)gsrc;
+
+	g_free(src->buf);
+}
+
 /**
  * hinawa_snd_unit_listen:
  * @self: A #HinawaSndUnit
@@ -401,10 +403,11 @@ void hinawa_snd_unit_listen(HinawaSndUnit *self, GError **exception)
 		.prepare	= prepare_src,
 		.check		= check_src,
 		.dispatch	= dispatch_src,
-		.finalize	= NULL,
+		.finalize	= finalize_src,
 	};
 	HinawaSndUnitPrivate *priv;
 	void *buf;
+	unsigned int len;
 	GSource *src;
 
 	g_return_if_fail(HINAWA_IS_SND_UNIT(self));
@@ -414,7 +417,8 @@ void hinawa_snd_unit_listen(HinawaSndUnit *self, GError **exception)
 	 * MEMO: allocate one page because we cannot assume the size of
 	 * transaction frame.
 	 */
-	buf = g_malloc(getpagesize());
+	len = sysconf(_SC_PAGESIZE);
+	buf = g_malloc(len);
 	if (buf == NULL) {
 		raise(exception, ENOMEM);
 		return;
@@ -431,23 +435,21 @@ void hinawa_snd_unit_listen(HinawaSndUnit *self, GError **exception)
 	g_source_set_priority(src, G_PRIORITY_HIGH_IDLE);
 	g_source_set_can_recurse(src, TRUE);
 
+	((SndUnitSource *)src)->len = len;
+	((SndUnitSource *)src)->buf = buf;
 	((SndUnitSource *)src)->unit = self;
-	priv->src = (SndUnitSource *)src;
-	priv->buf = buf;
-	priv->len = getpagesize();
-
 	((SndUnitSource *)src)->tag =
-		hinawa_context_add_src(HINAWA_CONTEXT_TYPE_SND, src, priv->fd,
-				       G_IO_IN, exception);
+				g_source_add_unix_fd(src, priv->fd, G_IO_IN);
+
+	hinawa_context_add_src(HINAWA_CONTEXT_TYPE_SND, src, exception);
 	if (*exception != NULL) {
-		g_free(buf);
 		g_source_destroy(src);
-		priv->buf = NULL;
-		priv->len = 0;
 		priv->src = NULL;
 		return;
 	}
 
+	priv->src = (SndUnitSource *)src;
+
 	hinawa_fw_unit_listen(&self->parent_instance, exception);
 	if (*exception != NULL) {
 		hinawa_snd_unit_unlisten(self);
@@ -472,23 +474,26 @@ void hinawa_snd_unit_listen(HinawaSndUnit *self, GError **exception)
 void hinawa_snd_unit_unlisten(HinawaSndUnit *self)
 {
 	HinawaSndUnitPrivate *priv;
+	GSource *gsrc;
 
 	g_return_if_fail(HINAWA_IS_SND_UNIT(self));
 	priv = hinawa_snd_unit_get_instance_private(self);
 
 	if (priv->src == NULL)
 		return;
+	gsrc = (GSource *)priv->src;
 
 	if (priv->streaming)
 		ioctl(priv->fd, SNDRV_FIREWIRE_IOCTL_UNLOCK, NULL);
 
-	hinawa_context_remove_src(HINAWA_CONTEXT_TYPE_SND, (GSource *)priv->src);
+	g_source_remove_unix_fd(gsrc, priv->src->tag);
+
+	hinawa_context_remove_src(HINAWA_CONTEXT_TYPE_SND, gsrc);
+
+	g_source_destroy(gsrc);
+
 	g_free(priv->src);
 	priv->src = NULL;
 
-	g_free(priv->buf);
-	priv->buf = NULL;
-	priv->len = 0;
-
 	hinawa_fw_unit_unlisten(&self->parent_instance);
 }
